"use strict";

function AppStorage(applicationName)
{
    var prefix = (applicationName ? applicationName + "." : "");
    
    this.isLocalStorageSupported = (('localStorage' in window) &&
        window['localStorage']);
    this.isLocalStorageSupported = this.isLocalStorageSupported ? true : false;
        
    this.setValue = function(key, val)
    {
        if (this.isLocalStorageSupported)
            localStorage.setItem(prefix + key, JSON.stringify(val));
        return this;
    };    
    
    this.getValue = function(key)
    {
        if (this.isLocalStorageSupported)
            return JSON.parse(localStorage.getItem(prefix + key));
        else
            return null;
    };
    
    this.removeValue = function(key)
    {
        if (this.isLocalStorageSupported)
            localStorage.removeItem(prefix + key);
        return this;
    };
    
    this.removeAll = function()
    {
        var keys = this.getKeys();
        for (var i in keys){
            this.removeValue(keys[i]);
        }
        return this;
    };
    
    this.getKeys = function(filter)
    {
        var keys = [];
        if (this.isLocalStorageSupported){
            for (var key in localStorage){
                if (isAppKey(key)){
                    // Remove the prefix from the key
                    if (prefix) key = key.slice(prefix.length);
                    // Check the filter
                    if (!filter || filter(key)){
                        keys.push(key);
                    }
                }
            }
        }
        return keys;
    };
    
    /*
    this.containsKey = function(key)
    {
        return this.getValue(key) !== null;
    };
    */
    
    this.containsKey = function(key)
    {
        if (prefix + key in localStorage)
            return true;
        return false;
    };
    
    function isAppKey(key)
    {
        if (prefix){
            return key.indexOf(prefix) === 0;
        }
        return true;
    };
        
    this.start = function()
    {
        console.log(prefix);
        console.log("local storage: ", this.isLocalStorageSupported);

        this.setValue("test_key_01", "test_value_01");
        console.log("getValue result: ", this.getValue("test_key_01"));
        
        // begin removeValue test
        console.log(localStorage);
        this.removeValue("test_key_01");         
        console.log("after remove: ", localStorage);
        
        // begin removeAll test
        this.setValue("test_key_02", "test_value_02");
        this.setValue("test_key_03", "test_value_03");
        console.log(localStorage);
        this.removeAll();
        console.log("after remove all: ", localStorage);
        
        // begin containsKey test
        this.setValue("test_key_04", "test_value_04");
        console.log(localStorage);
        console.log(this.containsKey("test_key_04"));
        console.log(this.containsKey("test_key_05"));
        
        // clean local storage
        this.removeAll();
        console.log(localStorage);
    };
}

function MyApp()
{
  var version = "v1.0",
      appStorage = new AppStorage("taskApplication");
    
  function setStatus(status)
  {
    $("#app>footer").text(status); 
  }
  
  function saveTaskList()
  {
    var tasks = [];
    $("#task-list .task span.task-name").each(function() {
      tasks.push($(this).text())
    });
    appStorage.setValue("taskList", tasks);
  }
  
  function loadTaskList()
  {
    var tasks = appStorage.getValue("taskList");
    if (tasks)
    {
      for (var i in tasks) 
      {
        addTaskItem(tasks[i]);
      }
    }
  }
  
  function addTask()
  {
    var taskName = $("#new-task-name").val();
    if (taskName)
    {
      addTaskItem(taskName);
      // Reset the text field
      $("#new-task-name").val("").focus();
    }
  }
  
  function addTaskItem(text)
  {
    var taskItem = $("#task-template .task").clone();
    
    $("span.task-name", taskItem).text(text);
    $("#task-list").append(taskItem);
    saveTaskList();
    
    $("button.delete", taskItem).click(function() {
      taskItem.remove();
      saveTaskList();
    });
    $("button.move-up", taskItem).click(function() {
      taskItem.insertBefore(taskItem.prev());
      saveTaskList();
    });
    $("button.move-down", taskItem).click(function() {
      taskItem.insertAfter(taskItem.next());
      saveTaskList();
    });
  }
  
  this.start = function()
  {
    $("#new-task-name").keypress(function(e) {
      if (e.which == 13) // Enter key
      {
        addTask();
        return false;
      }
    })
    .focus();

    $("#app>header").append(" "+version);
    loadTaskList();
    setStatus("Ready");
    
    // temporary for testing purposes
    /*
    $("#button-test").click(function() {
      saveTaskList();
      console.info(localStorage);
    });
    */
  };
}

$(function() {
  window.app = new MyApp();
  window.app.start();
});
